﻿Imports System.Drawing.Drawing2D

Module GraphPaperModule

    Friend LoudnessSelectedBand As Int32 = -1
    Friend LoudnessTestBand As Int32 = -1

    Friend ScaleMin_dB As Int32 = 26
    Friend ScaleMax_dB As Int32 = 125

    Friend DeltaSamples As Single
    Friend AutoMax As Boolean

    Friend DataArray1 As DataArray

    Friend GraphPaper1 As GraphPaper = New GraphPaper

    Friend Sub RedimDataArrayMax(ByVal n As Int32)
        DataArray1 = New DataArray(n)
    End Sub

    Friend Sub LimitDeltaSamples()
        If DataArray1 Is Nothing Then Return
        Dim MaxDelta As Single = DataArray1.GetNumEntries - 1
        If DeltaSamples > MaxDelta Then DeltaSamples = MaxDelta
        If DeltaSamples < 0 Then DeltaSamples = 0
    End Sub

    Friend Sub InitPictureboxImage(ByVal pbox As PictureBox)
        With pbox
            If .ClientSize.Width < 1 Or .ClientSize.Height < 1 Then Return
            If .Image Is Nothing OrElse .Image.Width <> .ClientSize.Width _
                                 OrElse .Image.Height <> .ClientSize.Height Then
                .Image = New Bitmap(.ClientRectangle.Width, _
                                    .ClientRectangle.Height, _
                                    Imaging.PixelFormat.Format24bppRgb)
            End If
        End With
    End Sub

    Friend Function MillisecToHourMinAndSec(ByVal millisec As Single) As String
        If millisec >= 3600000 Then
            Dim ts As TimeSpan = TimeSpan.FromMilliseconds(millisec)
            Return Math.Floor(ts.TotalHours).ToString() + "h " + _
                   Math.Floor(ts.Minutes).ToString() + "m " + _
                              ts.Seconds.ToString + "s"
        ElseIf millisec >= 60000 Then
            Dim ts As TimeSpan = TimeSpan.FromMilliseconds(millisec)
            Return Math.Floor(ts.TotalMinutes).ToString() + "m " _
                                    + ts.Seconds.ToString + "s"
        Else
            Return (millisec / 1000).ToString("0", GCI) + " Sec"
        End If
    End Function

End Module

Public Class DataArray

    Friend Structure DataSample
        Dim LAI As Single
        Dim LAF As Single
        Dim LAS As Single
        Dim Time As DateTime
        Sub New(ByVal _LAI As Single, _
                ByVal _LAF As Single, _
                ByVal _LAS As Single, _
                ByVal _time As DateTime)
            LAI = _LAI
            LAF = _LAF
            LAS = _LAS
            Time = _time
        End Sub
    End Structure

    Private m_ArrayMaxLength As Int32
    Private m_WriteIndex As Int32
    Private m_DataArray() As DataSample
    Private m_NumEntries As Int32

    Friend Sub New(ByVal ArrayMaxLength As Int32)
        m_ArrayMaxLength = ArrayMaxLength
        ReDim m_DataArray(m_ArrayMaxLength - 1)
        Flush()
    End Sub

    Friend Sub Flush()
        m_WriteIndex = 0
        m_NumEntries = 0
    End Sub

    Friend Function GetWriteIndex() As Int32
        Return m_WriteIndex
    End Function

    Friend Sub SetWriteIndex(ByVal WriteIndex As Int32)
        If WriteIndex < 0 Then WriteIndex = 0
        If WriteIndex > m_ArrayMaxLength - 1 Then WriteIndex = m_ArrayMaxLength - 1
        m_WriteIndex = WriteIndex
    End Sub

    Friend Sub SetNumEntries(ByVal n As Int32)
        If n < 0 Then n = 0
        If n > m_ArrayMaxLength - 1 Then n = m_ArrayMaxLength - 1
        m_NumEntries = n
    End Sub

    Friend Function GetNumEntries() As Int32
        Return m_NumEntries
    End Function

    Friend Function GetBuffer() As DataSample()
        Return m_DataArray
    End Function

    Friend Sub AddElement(ByVal LAI As Double, _
                          ByVal LAF As Double, _
                          ByVal LAS As Double, _
                          ByVal Time As DateTime)
        Dim ds As DataSample
        ds = New DataSample(CSng(LAI), CSng(LAF), CSng(LAS), Time)
        m_DataArray(m_WriteIndex) = ds
        m_WriteIndex += 1
        If m_WriteIndex > m_ArrayMaxLength - 1 Then m_WriteIndex = m_ArrayMaxLength - 1
        m_NumEntries += 1
        If m_NumEntries > m_ArrayMaxLength Then m_NumEntries = m_ArrayMaxLength
    End Sub

    Friend Sub FillWithInvalidData()
        Dim ds As DataSample = New DataSample(-999, -999, -999, DateTime.MinValue)
        For i As Int32 = 0 To m_ArrayMaxLength - 1
            m_DataArray(i) = ds
        Next
    End Sub

    Friend Function GetElement(ByVal Index As Int32) As DataSample
        If Index < 0 Then Index = 0
        If Index > m_ArrayMaxLength - 1 Then Index = m_ArrayMaxLength - 1
        Return m_DataArray(Index)
    End Function

End Class


Public Class GraphPaper

    Friend ScaleHorizontal As Single = 1

    Friend PB_TimeGraph As PictureBox
    Friend PB_FrequencyGraph As PictureBox

    Private BackColor As Color = Color.FromArgb(250, 240, 210) ' 240, 230, 200
    Private Pen_Scale1 As Pen = New Pen(Color.FromArgb(150, 120, 120))
    Private Pen_Scale2 As Pen = New Pen(Color.FromArgb(200, 200, 200))

    Private Pen_CenterDivision As Pen = New Pen(Color.FromArgb(255, 255, 255, 100), 11)
    Private Pen_Hours As Pen = New Pen(Color.FromArgb(180, 100, 100), 5)
    Private Pen_Minutes As Pen = New Pen(Color.FromArgb(120, 180, 120), 3)
    Private Pen_Seconds As Pen = New Pen(Color.FromArgb(150, 150, 220))
    Private Pen_TimeGap As Pen = New Pen(Color.FromArgb(150, 150, 150), 20)

    Private Pen_Trace1 As Pen = New Pen(Color.FromArgb(150, 0, 0), 2)
    Private Pen_Trace2 As Pen = New Pen(Color.FromArgb(60, 60, 180), 2)
    Private Pen_Trace3 As Pen = New Pen(Color.FromArgb(0, 150, 0), 2)

    Private Brush_Trace1 As Brush = New SolidBrush(Color.FromArgb(150, 0, 0))
    Private Brush_Trace2 As Brush = New SolidBrush(Color.FromArgb(60, 60, 180))
    Private Brush_Trace3 As Brush = New SolidBrush(Color.FromArgb(0, 150, 0))

    Private Brush_LoudnessBlue As Brush = New SolidBrush(Color.FromArgb(70, 80, 240))

    Private Brush_Text As Brush = New SolidBrush(Color.FromArgb(50, 50, 50))
    Private Brush_HilightBack As Brush = New SolidBrush(Color.FromArgb(220, 255, 255, 120))
    Private Brush_HilightText As Brush = New SolidBrush(Color.FromArgb(50, 50, 50))

    Private Font_Text As Font = New Font("Arial", 10, FontStyle.Bold)
    Private Font_Scale As Font = New Font("Arial", 9, FontStyle.Bold)
    Private Font_Labels As Font = New Font("Arial", 10, FontStyle.Bold)

    ' ==============================================================================
    '  UpdateDeltaSamples by DeltaPixels
    ' ==============================================================================
    Friend Sub UpdateWithMousePressed(ByVal DeltaX As Int32, ByVal DeltaY As Int32, ByVal mouseX As Int32, ByVal mouseY As Int32)
        If mouseX < BorderLeft Then
            If mouseY < PB_TimeGraph.ClientSize.Height * 0.5 Then
                ScaleMax_dB += DeltaY
                Form1.txt_SpecMaxDb.NumericValueInteger = ScaleMax_dB
                ScaleMax_dB = Form1.txt_SpecMaxDb.NumericValueInteger
            Else
                ScaleMin_dB += DeltaY
                Form1.txt_SpecMinDb.NumericValueInteger = ScaleMin_dB
                ScaleMin_dB = Form1.txt_SpecMinDb.NumericValueInteger
            End If
        ElseIf mouseY > hpix - BorderDown Then
            Dim xs As Single = ScaleHorizontal
            xs = xs * (1.0F - DeltaX * 0.01F)
            If xs < 0.1 Then xs = 0.1
            If xs > 2000 Then xs = 2000
            SetHorizontalScale(xs)
        Else
            DeltaSamples += DeltaX * ScaleHorizontal
            LimitDeltaSamples()
        End If
    End Sub

    Friend Sub UpdateScalesWithMouseWheel(ByVal mouseX As Int32, ByVal mouseY As Int32, ByVal delta As Int32)
        If mouseX < BorderLeft Then
            If mouseY < hpix / 2 Then
                ScaleMax_dB += delta * 10
                Form1.txt_SpecMaxDb.NumericValueInteger = ScaleMax_dB
                ScaleMax_dB = Form1.txt_SpecMaxDb.NumericValueInteger
            Else
                ScaleMin_dB += delta * 10
                Form1.txt_SpecMinDb.NumericValueInteger = ScaleMin_dB
                ScaleMin_dB = Form1.txt_SpecMinDb.NumericValueInteger
            End If
        Else
            Dim xs As Single = ScaleHorizontal
            xs = xs * (1.0F - delta * 0.3F)
            If xs < 0.1 Then xs = 0.1
            If xs > 2000 Then xs = 2000
            SetHorizontalScale(xs)
            LimitDeltaSamples()
        End If
    End Sub

    Friend Sub ShowTimePoint(ByVal time As DateTime)
        For i As Int32 = 0 To DataArray1.GetNumEntries - 1
            If DataArray1.GetElement(i).Time.Subtract(time).TotalMilliseconds >= 0 Then
                'Form1.Text = i.ToString
                DeltaSamples = DataArray1.GetNumEntries - i - 1
                UpdateTimeGraph()
                Exit For
            End If
        Next
    End Sub

    ' ==============================================================================
    '  RESIZE HORIZONTAL SCALE
    ' ==============================================================================
    Private ResizeScale_InitialWidth As Int32
    Private ResizeScale_InitialScale As Single

    Friend Sub SetHorizontalScale(ByVal hscale As Single)
        ScaleHorizontal = hscale
        ResizeScale_InitialScale = ScaleHorizontal
        If PB_TimeGraph.Image Is Nothing Then
            ResizeScale_InitialWidth = PB_TimeGraph.ClientSize.Width - BorderLeft
        Else
            ResizeScale_InitialWidth = PB_TimeGraph.Image.Width - BorderLeft
        End If
    End Sub

    Friend Sub ResizeEvent()
        If PB_TimeGraph.Image Is Nothing Then Return
        UpdateTimeGraph()
        ScaleHorizontal = ResizeScale_InitialScale * ResizeScale_InitialWidth / (PB_TimeGraph.Image.Width - BorderLeft)
        UpdateTimeGraph()
    End Sub

    ' ---------------------------------------------------------- set the scale to see all the buffer 
    Friend Sub ScaleHorizontal_SetToBuffer(ByVal NumEntries As Int32)
        Dim s As Single = CSng(NumEntries / (wpix - 80))
        If Single.IsNaN(s) Then s = 5000
        If s > 5000 Then s = 5000
        If s < 0.01 Then s = 0.01
        DeltaSamples = NumEntries \ 2
        SetHorizontalScale(s)
    End Sub

    ' ---------------------------------------------------------- set the scale to see the last seconds
    Friend Sub ScaleHorizontal_SetToLastSeconds()
        SetHorizontalScale(0.5)
        DeltaSamples = 0
    End Sub


    ' ==============================================================================
    '  TIME GRAPH
    ' ==============================================================================
    Private hpix As Int32
    Private wpix As Int32

    Private BorderLeft As Int32 = 30
    Private BorderUp As Int32 = 2
    Private BorderDown As Int32 = 20

    Friend Function DecibelToY(ByVal value As Double) As Single
        Return CSng(hpix - BorderDown - (value - ScaleMin_dB) / (ScaleMax_dB - ScaleMin_dB) * (hpix - BorderUp - BorderDown))
    End Function
    Public Function YToDecibel(ByVal Y As Int32) As Double
        Return (-Y + hpix - BorderDown) / (hpix - BorderUp - BorderDown) * (ScaleMax_dB - ScaleMin_dB) + ScaleMin_dB
    End Function

    Friend Sub UpdateTimeGraph()
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        ' --------------------------------------------------------------------- 
        If Not Form1.GroupBox1.Enabled Then Return
        If Not PB_TimeGraph.Visible Then Return
        InitPictureboxImage(PB_TimeGraph)
        If PB_TimeGraph.Image Is Nothing Then Return
        ' --------------------------------------------------------------------- 
        Dim v1, v2, v3 As Single
        Dim i As Int32
        Dim y1, y2, y3 As Single
        Dim oldx As Int32
        Dim oldy1, oldy2, oldy3 As Single
        Dim oldTime As DateTime = DateTime.MinValue
        Dim strTime As String = ""
        Dim MaxTimeGap As Single = 2 + ScaleHorizontal * RowTimeMS / 500.0F
        Dim xmid As Single
        ' ---------------------------------------------------------------------
        hpix = PB_TimeGraph.Image.Height
        wpix = PB_TimeGraph.Image.Width
        xmid = CInt(BorderLeft + (wpix - BorderLeft) / 2.0F)
        If hpix < 1 Or wpix < 1 Then Return
        ' ---------------------------------------------------------------------
        Dim g As Graphics = Graphics.FromImage(PB_TimeGraph.Image)
        g.Clear(BackColor)
        If ScaleMin_dB >= ScaleMax_dB Then
            g.DrawString(" INVALID PARAMS", Font_Scale, Brushes.Black, 24, 24)
            Return
        End If
        ' --------------------------------------------------------------------- file not found
        If DataArray1 Is Nothing Then
            g.DrawString(" FILE NOT FOUND", Font_Scale, Brushes.Black, 24, 24)
            PB_TimeGraph.Refresh()
            UpdateFrequencyGraph(-1)
            Return
        End If
        ' --------------------------------------------------------------------- array index with DeltaSamples
        Dim ArrayReadIndexCentral As Int32
        Dim ArrayReadIndexLeft As Single
        Dim ArrayReadIndexInt As Int32
        Static oldArrayReadIndexInt As Int32
        Dim CentralSample As DataArray.DataSample = Nothing
        Dim ClearanceLeft As Int32 = 2
        '
        ' --------------------------------------------------------------------- ArrayReadIndexCentral and CentralSample
        ArrayReadIndexCentral = CInt(DataArray1.GetNumEntries - 1 - DeltaSamples)
        CentralSample = DataArray1.GetElement(ArrayReadIndexCentral)
        '
        ' --------------------------------------------------------------------- ArrayReadIndexLeft
        ArrayReadIndexLeft = ArrayReadIndexCentral - ScaleHorizontal * (xmid - BorderLeft - ClearanceLeft)
        '
        CentralSample = DataArray1.GetElement(ArrayReadIndexCentral)
        '
        ' --------------------------------------------------------------------- stabilize and suppress aliasing effect
        ArrayReadIndexLeft = CInt(ArrayReadIndexLeft / ScaleHorizontal) * ScaleHorizontal
        '
        ' --------------------------------------------------------------------- draw center division
        g.DrawLine(Pen_CenterDivision, xmid - 1, 18, xmid - 1, hpix - BorderDown)
        '
        ' --------------------------------------------------------------------- draw scale y
        For i = ScaleMin_dB To ScaleMax_dB
            y1 = DecibelToY(i)
            If i Mod 10 = 0 Then
                g.DrawLine(Pen_Scale1, BorderLeft, y1, wpix + 1, y1)
                Dim w As Single = g.MeasureString(i.ToString, Font_Scale).Width
                g.DrawString(i.ToString, Font_Scale, Brushes.Black, 26 - w, y1 - 8)
            ElseIf i Mod 5 = 0 Then
                g.DrawLine(Pen_Scale2, BorderLeft, y1, wpix + 1, y1)
            End If
        Next
        ' --------------------------------------------------------------------- draw borders
        g.DrawLine(Pen_Scale1, BorderLeft + 2, BorderUp, BorderLeft + 2, hpix - BorderDown)
        g.DrawLine(Pen_Scale1, BorderLeft, hpix - BorderDown, wpix + 1, hpix - BorderDown)
        ' --------------------------------------------------------------------- draw all waves
        Dim GraphUpdatedFlag As Boolean = False
        Dim MidPoint As Int32 = CInt(wpix / 2) - 1
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        oldx = -1
        oldy1 = 0
        oldy2 = 0
        oldy3 = 0
        For ix As Int32 = BorderLeft + ClearanceLeft To wpix + CInt(1 / ScaleHorizontal)
            ' ----------------------------------------------------------------- 
            ArrayReadIndexLeft += ScaleHorizontal
            ArrayReadIndexInt = CInt(ArrayReadIndexLeft - 0.5)
            
            ' 
            ' Comparing ArrayReadIndexInt with 1 the first sample will be not visible 
            If ArrayReadIndexInt < 0 Then Continue For
            '

            If ArrayReadIndexInt > DataArray1.GetNumEntries - 1 Then
                ArrayReadIndexInt = DataArray1.GetNumEntries - 1
            End If
            ' ----------------------------------------------------------------- 
            With DataArray1.GetElement(ArrayReadIndexInt)
                v1 = .LAI
                v2 = .LAF
                v3 = .LAS
                ' ------------------------------------------------------------- Update Frequency Graph
                If Not GraphUpdatedFlag AndAlso .Time = CentralSample.Time Then
                    GraphUpdatedFlag = True
                    UpdateFrequencyGraph(ArrayReadIndexInt)
                End If
                ' ------------------------------------------------------------- draw Hour/Min/Sec divisions
                Dim ytxt As Single = hpix - BorderDown + 4
                If oldTime > DateTime.MinValue Then
                    If ScaleHorizontal > 20 Then
                        If .Time.Hour <> oldTime.Hour Then
                            g.DrawLine(Pen_Hours, ix, 0, ix, hpix - BorderDown)
                            If ScaleHorizontal > 1000 Then
                                g.DrawString(.Time.Hour.ToString, Font_Scale, Brush_Text, ix - 9, ytxt)
                            Else
                                g.DrawString(.Time.Hour.ToString + "h", Font_Scale, Brush_Text, ix - 9, ytxt)
                            End If
                        End If
                        If .Time.Minute \ 10 <> oldTime.Minute \ 10 And ScaleHorizontal < 300 Then
                            g.DrawLine(Pen_Minutes, ix, BorderUp, ix, hpix - BorderDown)
                        End If
                    ElseIf ScaleHorizontal > 0.7 Then
                        If .Time.Minute <> oldTime.Minute Then
                            g.DrawLine(Pen_Minutes, ix, 0, ix, hpix - BorderDown)
                            g.DrawString(.Time.Minute.ToString + "m", Font_Scale, Brush_Text, ix - 9, ytxt)
                        End If
                        If .Time.Second \ 10 <> oldTime.Second \ 10 And ScaleHorizontal < 10 Then
                            g.DrawLine(Pen_Seconds, ix, BorderUp, ix, hpix - BorderDown)
                        End If
                    Else
                        If .Time.Second <> oldTime.Second Then
                            g.DrawLine(Pen_Seconds, ix, 0, ix, hpix - BorderDown)
                            g.DrawString(.Time.Second.ToString.PadLeft(2), Font_Scale, Brush_Text, ix - 8, ytxt)
                        End If
                    End If
                    ' ------------------------------------------------------------- draw TIME-GAP division
                    If Math.Abs(.Time.Subtract(oldTime).TotalSeconds) > MaxTimeGap Then
                        g.DrawLine(Pen_TimeGap, ix, BorderUp, ix, hpix)
                        g.RotateTransform(-90)
                        g.DrawString("TimeGap", Font_Scale, Brushes.White, -(hpix - 8 - BorderDown), ix - 7)
                        g.ResetTransform()
                    End If
                End If
                oldTime = .Time
            End With
            ' ----------------------------------------------------------------- connect samples with diagonal line
            If ArrayReadIndexInt = oldArrayReadIndexInt Then Continue For
            oldArrayReadIndexInt = ArrayReadIndexInt
            ' ----------------------------------------------------------------- exclude blank area
            If v1 = -999 Or v2 = -999 Or v3 = -999 Then Continue For
            If Single.IsInfinity(v1) Or Single.IsInfinity(v2) Or Single.IsInfinity(v3) Then Continue For
            ' ----------------------------------------------------------------- y positions
            y1 = DecibelToY(v1)
            y2 = DecibelToY(v2)
            y3 = DecibelToY(v3)
            ' ----------------------------------------------------------------- draw 
            If oldx >= 0 Then
                g.DrawLine(Pen_Trace1, oldx, oldy1, ix, y1)
                g.DrawLine(Pen_Trace2, oldx, oldy2, ix, y2)
                g.DrawLine(Pen_Trace3, oldx, oldy3, ix, y3)
            End If
            ' ----------------------------------------------------------------- set old values
            oldx = ix
            oldy1 = y1
            oldy2 = y2
            oldy3 = y3
        Next
        ' --------------------------------------------------------------------- draw central time text
        strTime = CentralSample.Time.ToString("dd MMM yyyy HH:mm:ss + fff") + "mS"
        DrawHilightedString(g, strTime, Brush_HilightText, _
                                xmid - 39, 0, HorizontalAlignment.Center)
        ' --------------------------------------------------------------------- draw values
        DrawHilightedString(g, "LAi " + CentralSample.LAI.ToString("0.0", GCI), Brush_Trace1, BorderLeft + 10, 2)
        DrawHilightedString(g, "LAf " + CentralSample.LAF.ToString("0.0", GCI), Brush_Trace2, wpix - 135, 2)
        DrawHilightedString(g, "LAs " + CentralSample.LAS.ToString("0.0", GCI), Brush_Trace3, wpix - 64, 2)
        ' --------------------------------------------------------------------- show
        PB_TimeGraph.Invalidate()
        'ReportString = "Scope time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
        'Form1.Text = "Scope time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
    End Sub

    Private Sub DrawHilightedString(ByVal g As Graphics, _
                                    ByVal str As String, _
                                    ByVal br As Brush, _
                                    ByVal x As Single, _
                                    ByVal y As Single, _
                                    Optional ByVal align As HorizontalAlignment = HorizontalAlignment.Left)
        Dim sz As SizeF = g.MeasureString(str, Font_Labels, 999)
        Select Case align
            Case HorizontalAlignment.Left
            Case HorizontalAlignment.Center : x -= sz.Width / 2.0F
            Case HorizontalAlignment.Right : x -= sz.Width
        End Select
        g.FillRectangle(Brush_HilightBack, x - 6, y - 2, sz.Width + 6, sz.Height + 4)
        g.DrawString(str, Font_Labels, br, x, y)
    End Sub



    Private m_fillbrush As LinearGradientBrush = Nothing
    Private m_backbrush As Brush

    Friend Sub InitFrequencyBars()
        Dim hpix As Int32 = PB_FrequencyGraph.ClientSize.Height - 1
        Dim wpix As Int32 = PB_FrequencyGraph.ClientSize.Width - 1
        Dim r As RectangleF = New RectangleF(0, 0, CSng(wpix / NumBands), hpix)
        m_fillbrush = New LinearGradientBrush(r, Color.Black, Color.Black, 0)
        Dim myBlend As ColorBlend = New ColorBlend()
        myBlend.Positions = New Single() {0.0F, 0.5F, 0.8F, 0.95F, 1.0F}
        myBlend.Colors = New Color() {Color.FromArgb(200, 255, 180), _
                                      Color.FromArgb(80, 150, 50), _
                                      Color.FromArgb(80, 150, 0), _
                                      Color.FromArgb(80, 80, 80), _
                                      Color.FromArgb(50, 50, 50)}
        m_fillbrush.InterpolationColors = myBlend
        m_backbrush = New SolidBrush(PB_FrequencyGraph.BackColor)
    End Sub


    Friend Sub UpdateFrequencyGraph(ByVal index As Int32)
        ' ---------------------------------------------------------------------
        If Not PB_FrequencyGraph.Visible Then Return
        InitPictureboxImage(PB_FrequencyGraph)
        If PB_FrequencyGraph.Image Is Nothing Then Return
        ' --------------------------------------------------------------------- 
        Dim hpix As Int32 = PB_FrequencyGraph.Image.Height - 1
        Dim wpix As Int32 = PB_FrequencyGraph.Image.Width - 1
        ' ---------------------------------------------------------------------
        If hpix < 1 Or wpix < 1 Then Return
        ' ---------------------------------------------------------------------
        Dim rowsize As Single = CSng(wpix / NumBands)
        ' ---------------------------------------------------------------------
        Dim g As Graphics = Graphics.FromImage(PB_FrequencyGraph.Image)
        g.Clear(Color.FromArgb(220, 220, 0))
        ' ---------------------------------------------------------------------
        If index < 0 Then
            g.Clear(Color.FromArgb(255, 245, 220))
            g.DrawString(" FILE NOT FOUND", Font_Scale, Brushes.Black, 24, 24)
            PB_FrequencyGraph.Refresh()
            Return
        End If
        ' --------------------------------------------------------------------- get data line
        Dim line As String = GetFileRow(index)
        ' --------------------------------------------------------------------- read values from line
        Dim BandsValues(NumBands - 1) As Single
        Dim sa() As String = Split(line, vbTab)
        If sa.Length <= NumBands Then Return
        For i As Int32 = 0 To NumBands - 1
            BandsValues(i) = CSng(Val(sa(i + 1)))
        Next
        ' --------------------------------------------------------------------- DRAW BANDS
        Dim s As String
        Dim x As Single = 0
        Dim h As Single = 0
        Dim sz As SizeF
        Dim fs As Int32 = wpix \ 80
        If fs > 11 Then fs = 12
        If fs < 6 Then fs = 6
        Dim dx As Int32 = CInt(wpix / NumBands / 2) - fs \ 2 - 1
        Dim freq As Double
        'Dim klog As Double = (MaxFreq / MinFreq) ^ (1 / (NumBands - 1))
        Dim f As Font = New Font("Lucida console", fs, FontStyle.Bold)

        For i As Int32 = 0 To NumBands - 1
            ' ----------------------------------------------------------------- calc bands height
            h = (BandsValues(i) - ScaleMin_dB) / (ScaleMax_dB - ScaleMin_dB)
            h = hpix - h * hpix
            If h < 0 Then h = 0
            ' ----------------------------------------------------------------- draw green bands
            g.FillRectangle(m_backbrush, x + 2, 1, rowsize - 3, h - 1)
            g.FillRectangle(m_fillbrush, x + 2, h, rowsize - 3, hpix - 1)
            ' ----------------------------------------------------------------- print frequencies
            g.RotateTransform(-90)
            freq = StandardFreq(i)
            s = freq.ToString("0")
            sz = g.MeasureString(s, f, 999)
            If hpix - h > sz.Width + 2 Then
                g.DrawString(s, f, Brushes.White, -hpix + 2, x + dx)
            End If
            ' ----------------------------------------------------------------- print band values
            g.DrawString(BandsValues(i).ToString("0.0", GCI), f, Brushes.Black, -h + 2, x + dx)
            ' ----------------------------------------------------------------- print test value
            s = ""
            If LoudnessTestBand >= 0 AndAlso i = LoudnessTestBand + 1 Then
                s = (BandsValues(i) - BandsValues(i - 1)).ToString("0.0", GCI)
            End If
            If LoudnessTestBand < 31 AndAlso i = LoudnessTestBand - 1 Then
                s = (BandsValues(i) - BandsValues(i + 1)).ToString("0.0", GCI)
            End If
            If s <> "" Then
                sz = g.MeasureString(s, f, 999)
                g.DrawString(s, f, Brushes.Yellow, -h - sz.Width - 8, x + dx)
            End If
            ' -----------------------------------------------------------------
            g.ResetTransform()
            x += rowsize
        Next
        ' --------------------------------------------------------------------- DRAW LOUDNESS BANDS
        If AutoMax Then
            Dim maxvalue As Double = -999
            For i As Int32 = 0 To NumBands - 3
                If BandsValues(i) > maxvalue Then
                    maxvalue = BandsValues(i)
                    LoudnessSelectedBand = i
                End If
            Next
        End If
        If LoudnessSelectedBand >= 0 And LoudnessSelectedBand <= 28 Then
            Dim Ln As Double
            Dim Lp As Single
            Ln = LoudnessLevel(BandsValues(LoudnessSelectedBand), LoudnessSelectedBand)
            x = 0
            For i As Int32 = 0 To NumBands - 3
                Lp = CSng(PressureLevel(Ln, i))
                h = (Lp - ScaleMin_dB) / (ScaleMax_dB - ScaleMin_dB)
                h = hpix - h * hpix
                If i = LoudnessSelectedBand Then
                    g.FillRectangle(Brushes.Gold, x + 2, h - 2, rowsize - 3, 5)
                Else
                    If Lp > BandsValues(i) Then
                        g.FillRectangle(Brush_LoudnessBlue, x + 2, h - 2, rowsize - 3, 5)
                    Else
                        g.FillRectangle(Brushes.Red, x + 2, h - 2, rowsize - 3, 5)
                    End If
                End If
                x += rowsize
            Next
        End If
        ' ---------------------------------------------------------------------
        PB_FrequencyGraph.Refresh()
    End Sub

    Friend Sub SelectBand(ByVal x As Single)
        If PB_FrequencyGraph.Image Is Nothing Then Return
        Dim wpix As Int32 = PB_FrequencyGraph.Image.Width - 1
        Dim rowsize As Single = CSng(wpix / NumBands)
        Dim b As Int32 = CInt(Math.Floor(x / rowsize))
        LoudnessSelectedBand = b
        UpdateTimeGraph()
        'Form1.Text = LoudnessSelectedBand.ToString
    End Sub

    Friend Sub TestBand(ByVal x As Single)
        If Not Form1.GroupBox1.Enabled Then Return
        If x < 0 Then
            LoudnessTestBand = -1
            UpdateTimeGraph()
            Return
        End If
        If PB_FrequencyGraph.Image Is Nothing Then Return
        Dim wpix As Int32 = PB_FrequencyGraph.Image.Width - 1
        Dim rowsize As Single = CSng(wpix / NumBands)
        Dim b As Int32 = CInt(Math.Floor(x / rowsize))
        If b <> LoudnessTestBand Then
            LoudnessTestBand = b
            UpdateTimeGraph()
        End If
        'Form1.Text = LoudnessSelectedBand.ToString
    End Sub


    Friend Sub ClearGraphs()
        ' --------------------------------------------------------------------- 
        InitPictureboxImage(PB_TimeGraph)
        If PB_TimeGraph.Image Is Nothing Then Return
        If PB_TimeGraph.Image.Height < 1 Then Return
        If PB_TimeGraph.Image.Width < 1 Then Return
        Graphics.FromImage(PB_TimeGraph.Image).Clear(BackColor)
        ' ---------------------------------------------------------------------
        InitPictureboxImage(PB_FrequencyGraph)
        If PB_FrequencyGraph.Image Is Nothing Then Return
        If PB_FrequencyGraph.Image.Height < 1 Then Return
        If PB_FrequencyGraph.Image.Width < 1 Then Return
        Graphics.FromImage(PB_FrequencyGraph.Image).Clear(BackColor)
        ' ---------------------------------------------------------------------
        PB_TimeGraph.Refresh()
        PB_FrequencyGraph.Refresh()
    End Sub

End Class
