﻿
Imports System.Net
Imports System.Net.WebRequestMethods
Imports System.IO

Module FtpUpload

    Friend Working As Boolean = False
    Friend AbortFlag As Boolean
    Friend Delegate Sub ProgressCallBack(ByVal params As ProgressParams)

    Friend Structure ProgressParams
        Dim progress As Double
        Dim fileLength As Long
        Dim bytesRead As Int32
        Dim bytesTransferred As Long
        Dim secondsTotal As Double
        Dim secondsElapsed As Double
        Dim secondsRemaining As Double
        Dim kbps As Double
        Dim timeStart As Date
        Dim elapsedTime As Double
        Dim totalTime As Double
    End Structure


    ' ===================================================================================================
    '   FTP_Upload
    ' ===================================================================================================
    Friend Sub FTP_Upload(ByVal LocalFile As String, _
                          ByVal ServerFile As String, _
                          ByVal UserName As String, _
                          ByVal Password As String, _
                          Optional ByVal Callback As ProgressCallBack = Nothing)

        'If Working Then Exit Sub
        'Working = True
        'Application.UseWaitCursor = True
        'Cursor.Current = Cursors.WaitCursor

        Dim params As ProgressParams
        Dim FileStream As FileStream = Nothing
        Dim FtpStream As Stream = Nothing
        Dim FtpRequest As FtpWebRequest = Nothing

        With params

            Try
                ' --------------------------------------------------------------------- init buffer
                Dim bytesPerBlock As Integer = 4096
                Dim buffer(bytesPerBlock - 1) As Byte

                ' --------------------------------------------------------------------- FPT request (upload)
                FtpRequest = TryCast(WebRequest.Create(ServerFile), FtpWebRequest)
                FtpRequest.Method = WebRequestMethods.Ftp.UploadFile
                FtpRequest.Credentials = New NetworkCredential(UserName, Password)
                FtpRequest.UsePassive = True
                FtpRequest.UseBinary = True
                FtpRequest.KeepAlive = False ' <<< close the connection when done

                ' --------------------------------------------------------------------- open the FTP stream
                FtpStream = FtpRequest.GetRequestStream()

                ' --------------------------------------------------------------------- open the file stream
                FileStream = New FileStream(LocalFile, _
                                            FileMode.Open, _
                                            FileAccess.Read)

                ' --------------------------------------------------------------------- start the write loop
                AbortFlag = False
                .fileLength = FileStream.Length
                .bytesTransferred = 0
                .timeStart = Date.Now
                Dim oldseconds As Double = 0
                Do
                    ' ----------------------------------------------------------------- read from the file
                    .bytesRead = FileStream.Read(buffer, 0, buffer.Length)
                    '
                    If .bytesRead > 0 Then

                        ' ------------------------------------------------------------- update total bytes
                        .bytesTransferred += .bytesRead

                        ' ------------------------------------------------------------- write to stream
                        FtpStream.Write(buffer, 0, .bytesRead)

                        ' ------------------------------------------------------------- update progress
                        If Callback IsNot Nothing Then
                            .progress = (.bytesTransferred * 100) \ .fileLength
                            .secondsElapsed = (Date.Now - .timeStart).TotalSeconds
                            .secondsTotal += 0.3 * (.secondsElapsed * .fileLength / .bytesTransferred - .secondsTotal)
                            .secondsRemaining = .secondsTotal - .secondsElapsed
                            If .secondsTotal > 0 Then .kbps = 8 / 1000 * .fileLength / .secondsTotal
                            If .secondsElapsed >= oldseconds + 0.1 Then
                                oldseconds = .secondsElapsed
                                Callback(params)
                            End If
                        End If
                    End If

                    If AbortFlag Then FtpRequest.Abort()

                    Application.DoEvents()

                Loop While .bytesRead > 0

            Catch ex As Exception
                'MsgBox(ex.Message)
                If FtpRequest IsNot Nothing Then FtpRequest.Abort()
                Callback = Nothing

            Finally

                If Callback IsNot Nothing Then
                    .progress = 0
                    Callback(params)
                End If

                Try
                    If FileStream IsNot Nothing Then FileStream.Close()
                    If FtpStream IsNot Nothing Then FtpStream.Close()
                Catch
                End Try

            End Try

        End With

        'Application.UseWaitCursor = False
        'Cursor.Current = Cursors.Default
        'Working = False
    End Sub


    ' ==================================================================================================
    '  SEND REPORT FILE TO FTP
    ' ==================================================================================================
    Friend Sub SendPeriodicReport_FromLocalFileToFTP(ByVal fname As String)
        Try
            FtpUserName = Form1.txt_FtpUserName.Text
            FtpPassword = Form1.txt_FtpPassword.Text
            Dim f As String = Form1.txt_FtpFolder.Text + "/" + IO.Path.GetFileName(fname)
            FTP_Upload(fname, f, FtpUserName, FtpPassword)
        Catch
        End Try
    End Sub

    ' ==================================================================================================
    '  SEND SHORT REPORT IMAGE TO FTP
    ' ==================================================================================================
    Private ImageType As String = "jpg"
    Private ImageFileName As String = ""
    Private ImageFilePathAndName As String
    Private FtpFile As String
    Private FtpUserName As String
    Private FtpPassword As String

    Friend Sub SendShortReportImage_FromLocalFileToFTP(ByVal fname As String)
        ImageFilePathAndName = fname
        ImageFileName = IO.Path.GetFileName(fname)
        Try
            If Form1.chk_SendToFtp.Checked Then
                SendImageToFtp_ASYNC()
            End If
        Catch
        End Try
    End Sub

    Private Sub SendImageToFtp_ASYNC()
        ' --------------------------------------------------------- prepare vars before to start the thread
        FtpFile = Form1.txt_FtpFolder.Text & "/" & _
                  IO.Path.GetFileName(ImageFilePathAndName)
        FtpUserName = Form1.txt_FtpUserName.Text
        FtpPassword = Form1.txt_FtpPassword.Text
        ' --------------------------------------------------------- asynchronously start the Thread 
        Dim objThread As Threading.Thread
        objThread = New Threading.Thread(AddressOf SendImageToFtp)
        ' --------------------------------------------------------- make the thread as background thread.
        objThread.IsBackground = True
        ' --------------------------------------------------------- set the Priority of the thread.
        objThread.Priority = Threading.ThreadPriority.BelowNormal
        ' --------------------------------------------------------- start the thread.
        objThread.Start()
    End Sub

    Private Sub SendImageToFtp()
        FTP_Upload(ImageFilePathAndName, FtpFile, FtpUserName, FtpPassword)
    End Sub

End Module
