﻿Module Module_Loudness

    Friend StandardFreq() As Single = {20, 25, 31.5, 40, 50, 63, 80, 100, 125, 160, _
                                       200, 250, 315, 400, 500, 630, 800, 1000, 1250, 1600, _
                                       2000, 2500, 3150, 4000, 5000, 6300, 8000, 10000, 12500, 16000, _
                                       20000}

    Friend Enum LoudnessTypes
        ISO_A  ' ISO 226 1994
        ISO_B  ' ISO 226 2007
        ISO_C  ' 
    End Enum

    Friend LoudnessType As LoudnessTypes = LoudnessTypes.ISO_A

    Friend Function GetLoudnessName() As String
        Select Case LoudnessType
            Case LoudnessTypes.ISO_A : Return "ISO 226:1994"
            Case LoudnessTypes.ISO_B : Return "ISO 226:2007"
            Case LoudnessTypes.ISO_C
        End Select
        Return ""
    End Function


    ' --------------------------------------------------------------------------------
    ' Deriving loudness level, Ln, from frequency 
    ' (correspond to that af index i) of tonal component, CT
    ' --------------------------------------------------------------------------------
    Friend Function LoudnessLevel(ByVal Lp As Double, ByVal i As Int32) As Double
        Select Case LoudnessType
            Case LoudnessTypes.ISO_A : Return LoudnessLevel_A(Lp, i)
            Case LoudnessTypes.ISO_B : Return LoudnessLevel_B(Lp, i)
            Case LoudnessTypes.ISO_C
        End Select
        Return 0
    End Function

    ' ----------------------------------------------------------------------------
    '  Deriving sound pressure level from loudness level (iso226 sect 4.1)
    ' ----------------------------------------------------------------------------
    Friend Function PressureLevel(ByVal Ln As Double, ByVal j As Int32) As Double
        Select Case LoudnessType
            Case LoudnessTypes.ISO_A : Return PressureLevel_A(Ln, j)
            Case LoudnessTypes.ISO_B : Return PressureLevel_B(Ln, j)
            Case LoudnessTypes.ISO_C
        End Select
        Return 0
    End Function


    ' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    '   TYPE A - ISO226 -
    ' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Private Af_isoA() As Single = {2.347, 2.19, 2.05, 1.879, 1.724, 1.597, 1.512, 1.466, 1.426, 1.394, _
                                   1.372, 1.344, 1.304, 1.256, 1.203, 1.135, 1.062, 1.0, 0.967, 0.943, _
                                   0.932, 0.933, 0.937, 0.952, 0.974, 1.027, 1.135, 1.266, 1.501}

    Private Bf_isoA() As Single = {0.00561, 0.00527, 0.00481, 0.00404, 0.00338, 0.00286, 0.00259, 0.00257, 0.00256, 0.00255, _
                                   0.00254, 0.00248, 0.00229, 0.00201, 0.00162, 0.00111, 0.00052, 0.0, -0.00039, -0.00067, _
                                   -0.00092, -0.00105, -0.00104, -0.00088, -0.00055, 0.0, 0.00089, 0.00211, 0.00488}

    Private Tf_isoA() As Single = {74.3, 65.0, 56.3, 48.4, 41.7, 35.5, 29.8, 25.1, 20.7, 16.8, _
                                   13.8, 11.2, 8.9, 7.2, 6.0, 5.0, 4.4, 4.2, 3.7, 2.6, _
                                   1.0, -1.2, -3.6, -3.9, -1.1, 6.6, 15.3, 16.4, 11.6}

    Friend Function LoudnessLevel_A(ByVal Lp As Double, ByVal i As Int32) As Double
        Return 4.2 + Af_isoA(i) * (Lp - Tf_isoA(i)) / (1 + Bf_isoA(i) * (Lp - Tf_isoA(i)))
    End Function

    Friend Function PressureLevel_A(ByVal Ln As Double, ByVal j As Int32) As Double
        Dim a As Double
        a = Ln * (Bf_isoA(j) * Tf_isoA(j) - 1) - Tf_isoA(j) * (4.2 * Bf_isoA(j) + Af_isoA(j)) + 4.2
        Return a / (Ln * Bf_isoA(j) - 4.2 * Bf_isoA(j) - Af_isoA(j))
    End Function


    ' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    '   TYPE B - ISO226 
    ' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Private Af_isoB() As Single = {0.532, 0.506, 0.48, 0.455, 0.432, 0.409, 0.387, 0.367, 0.349, 0.33, _
                                   0.315, 0.301, 0.288, 0.276, 0.267, 0.259, 0.253, 0.25, 0.246, 0.244, _
                                   0.243, 0.243, 0.243, 0.242, 0.242, 0.245, 0.254, 0.271, 0.301}

    Private Lu_isoB() As Single = {-31.6, -27.2, -23.0, -19.1, -15.9, -13.0, -10.3, -8.1, -6.2, -4.5, _
                                   -3.1, -2.0, -1.1, -0.4, 0.0, 0.3, 0.5, 0.0, -2.7, -4.1, _
                                   -1.0, 1.7, 2.5, 1.2, -2.1, -7.1, -11.2, -10.7, -3.1}

    Private Tf_isoB() As Single = {78.5, 68.7, 59.5, 51.1, 44.0, 37.5, 31.5, 26.5, 22.1, 17.9, _
                                   14.4, 11.4, 8.6, 6.2, 4.4, 3.0, 2.2, 2.4, 3.5, 1.7, _
                                   -1.3, -4.2, -6.0, -5.4, -1.5, 6.0, 12.6, 13.9, 12.3}

    Friend Function LoudnessLevel_B(ByVal Lp As Double, ByVal i As Int32) As Double
        Dim bf As Double
        bf = (0.4 * 10 ^ ((Lp + Lu_isoB(i)) / 10 - 9)) ^ Af_isoB(i)
        bf -= (0.4 * 10 ^ ((Tf_isoB(i) + Lu_isoB(i)) / 10 - 9)) ^ Af_isoB(i)
        bf += 0.005135
        Return 40 * Math.Log10(bf) + 94
    End Function

    Friend Function PressureLevel_B(ByVal Ln As Double, ByVal j As Int32) As Double
        Dim a As Double
        a = 0.00447 * (10 ^ (0.025 * Ln) - 1.15) + (0.4 * 10 ^ (((Tf_isoB(j) + Lu_isoB(j)) / 10) - 9)) ^ Af_isoB(j)
        Return ((10 / Af_isoB(j)) * Math.Log10(a)) - Lu_isoB(j) + 94
    End Function


   
End Module
