﻿Module Calibrator

    Private CalibGen10(9) As Single
    Private CalibGen31(30) As Single

    Private CalibMic10(9) As Single
    Private CalibMic31(30) As Single

    ' =================================================================================================
    '   CALIBRATION ARRAYS - SAVE / LOAD
    ' =================================================================================================
    Friend Function GetCalibrationString() As String
        Dim s As String = ""
        s += "CalibGen10="
        For i As Int32 = 0 To 9
            s += " " + CalibGen10(i).ToString("0.0", GCI)
        Next
        s += vbCrLf
        s += "CalibGen31="
        For i As Int32 = 0 To 30
            s += " " + CalibGen31(i).ToString("0.0", GCI)
        Next
        s += vbCrLf
        s += "CalibMic10="
        For i As Int32 = 0 To 9
            s += " " + CalibMic10(i).ToString("0.0", GCI)
        Next
        s += vbCrLf
        s += "CalibMic31="
        For i As Int32 = 0 To 30
            s += " " + CalibMic31(i).ToString("0.0", GCI)
        Next
        Return s
    End Function

    Friend Sub SetCalibrationsFromString(ByVal calibName As String, ByVal s As String)
        Select Case calibName
            Case "CalibGen10" : SplitCalibRow(CalibGen10, s)
            Case "CalibGen31" : SplitCalibRow(CalibGen31, s)
            Case "CalibMic10" : SplitCalibRow(CalibMic10, s)
            Case "CalibMic31" : SplitCalibRow(CalibMic31, s)
        End Select
    End Sub

    Private Sub SplitCalibRow(ByRef CalibArray() As Single, ByVal s As String)
        Dim sa() As String = s.Split(" "c)
        If sa.Length = CalibArray.Length Then
            For i As Int32 = 0 To sa.Length - 1
                CalibArray(i) = CSng(Val(sa(i)))
            Next
        End If
    End Sub

    ' =================================================================================================
    '   CALIBRATION ARRAYS - INTERPOLATIONS
    ' =================================================================================================
    Friend Sub InterpolateCalibrations(ByVal InterpolationType As String)

        Select Case InterpolationType
            Case "10 to 31"

                'Dim f1, f2, fx, mic1, mic2, gen1, gen2 As Single
                'For j As Int32 = 0 To 8
                '    For i As Int32 = 0 To 30
                '        If i < 30 And RoundedFreq31(i) >= RoundedFreq10(j) And RoundedFreq31(i) < RoundedFreq10(j + 1) Then
                '            f1 = RoundedFreq10(j)
                '            f2 = RoundedFreq10(j + 1)
                '            fx = RoundedFreq31(i)
                '            mic1 = CalibMic10(j)
                '            mic2 = CalibMic10(j + 1)
                '            CalibMic31(i) = InterpolateAndExtrapolate(f1, f2, fx, mic1, mic2)
                '            gen1 = CalibGen10(j)
                '            gen2 = CalibGen10(j + 1)
                '            CalibGen31(i) = InterpolateAndExtrapolate(f1, f2, fx, gen1, gen2)

                '            Debug.Print("1  " + i.ToString + " " + j.ToString)

                '        ElseIf j = 8 And RoundedFreq31(i) > RoundedFreq10(j) Then 'in pratica 20 kHz
                '            f1 = RoundedFreq10(8)
                '            f2 = RoundedFreq10(9)
                '            fx = RoundedFreq31(i)
                '            mic1 = CalibMic10(8)
                '            mic2 = CalibMic10(9)
                '            CalibMic31(i) = InterpolateAndExtrapolate(f1, f2, fx, mic1, mic2)
                '            gen1 = CalibGen10(8)
                '            gen2 = CalibGen10(9)
                '            CalibGen31(i) = InterpolateAndExtrapolate(f1, f2, fx, gen1, gen2)

                '            Debug.Print("2  " + i.ToString + " " + 8.ToString)

                '        ElseIf i <= 1 And j < 2 Then ' in pratica 20 e 25 Hz
                '            f1 = RoundedFreq10(0)
                '            f2 = RoundedFreq10(1)
                '            fx = RoundedFreq31(i)
                '            mic1 = CalibMic10(0)
                '            mic2 = CalibMic10(1)
                '            CalibMic31(i) = InterpolateAndExtrapolate(f1, f2, fx, mic1, mic2)
                '            gen1 = CalibGen10(0)
                '            gen2 = CalibGen10(1)
                '            CalibGen31(i) = InterpolateAndExtrapolate(f1, f2, fx, gen1, gen2)

                '            Debug.Print("3  " + i.ToString + " " + 0.ToString)

                '        End If
                '    Next
                'Next

                Dim f1, f2, fx, mic1, mic2, gen1, gen2 As Single
                For i As Int32 = 0 To 30
                    ' ------------------------------------------------------------------
                    '  i = 0 1 2 3 4 5 6 7 8 9 10 11 12 13 ... 23 24 25 26 27 28 29 30
                    '  j = 0 0 0 0 0 1 1 1 2 2  2  3  3  3 ...  7  7  7  8  8  8  8  8 
                    ' ------------------------------------------------------------------
                    Dim j As Int32 = (i - 2) \ 3
                    If j > 8 Then j = 8
                    ' ---------------------------
                    f1 = RoundedFreq10(j)
                    f2 = RoundedFreq10(j + 1)
                    fx = RoundedFreq31(i)
                    mic1 = CalibMic10(j)
                    mic2 = CalibMic10(j + 1)
                    CalibMic31(i) = InterpolateAndExtrapolate(f1, f2, fx, mic1, mic2)
                    gen1 = CalibGen10(j)
                    gen2 = CalibGen10(j + 1)
                    CalibGen31(i) = InterpolateAndExtrapolate(f1, f2, fx, gen1, gen2)
                Next


            Case "31 to 10"
                Dim j As Int32 = 2
                For i As Int32 = 0 To 9
                    CalibMic10(i) = CalibMic31(j)
                    CalibGen10(i) = CalibGen31(j)
                    j += 3
                Next

        End Select
    End Sub

    ' =======================================================================================================
    '  Interpolation function
    ' -------------------------------------------------------------------------------------------------------
    '  Example (returned value x is 30)
    '   I1=1     ix=3     I2=5
    '   V1=10    x=??     V2=50
    ' =======================================================================================================
    Friend Function InterpolateAndExtrapolate(ByVal i1 As Single, ByVal i2 As Single, ByVal ix As Single, _
                                              ByVal v1 As Single, ByVal v2 As Single) As Single
        If i2 = i1 Then Return v1
        Return v1 + (v2 - v1) * (ix - i1) / (i2 - i1)
    End Function


    ' =================================================================================================
    '   CALIBRATION ARRAYS - READ WRITE VALUES
    ' =================================================================================================
    Friend Function GetGeneratorCalibrationValue(ByVal nbands As Int32, ByVal band As Int32) As Single
        Select Case nbands
            Case 10 : Return CalibGen10(band)
            Case 31 : Return CalibGen31(band)
            Case Else : Return 0
        End Select
    End Function

    Friend Sub SetGeneratorCalibrationValue(ByVal nbands As Int32, ByVal band As Int32, ByVal value As Single)
        Select Case nbands
            Case 10 : CalibGen10(band) = value
            Case 31 : CalibGen31(band) = value
        End Select
    End Sub

    Friend Function GetMicCalibrationValue(ByVal nbands As Int32, ByVal band As Int32) As Single
        Select Case nbands
            Case 10 : Return CalibMic10(band)
            Case 31 : Return CalibMic31(band)
            Case Else : Return 0
        End Select
    End Function

    Friend Sub SetMicCalibrationValue(ByVal nbands As Int32, ByVal band As Int32, ByVal value As Single)
        Select Case nbands
            Case 10 : CalibMic10(band) = value
            Case 31 : CalibMic31(band) = value
        End Select
    End Sub

End Module
