﻿
Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                               Optional ByVal Value As Double = Double.NaN, _
                               Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt, GCI)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function

    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Int32
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function



    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================

    Friend Sub Save_INI()
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("==============================================")
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            f.WriteLine(TabString("SplitterDistance", Form1.SplitContainer1.SplitterDistance))
            '
            f.WriteLine("")
            f.WriteLine(" Audio InOut")
            f.WriteLine("==============================================")
            f.WriteLine(TabString("SelectedAudioIn", Form1.SelectedAudioIn))
            f.WriteLine(TabString("SelectedAudioOut", Form1.SelectedAudioOut))
            '
            f.WriteLine("")
            f.WriteLine(" Controls")
            f.WriteLine("==============================================")
            f.WriteLine(TabString("Calibration", Form1.txt_Calibration.Text))
            f.WriteLine(TabString("RoundDisplayFreq", Form1.chk_RoundDisplayFreq.Checked))
            '
            f.WriteLine(TabString("GroupBox1_Height", Form1.GroupBox1.Height))
            f.WriteLine(TabString("GroupBox2_Height", Form1.GroupBox2.Height))
            f.WriteLine(TabString("GroupBox3_Height", Form1.GroupBox3.Height))
            f.WriteLine(TabString("GroupBox4_Height", Form1.GroupBox4.Height))
            f.WriteLine(TabString("GroupBox5_Height", Form1.GroupBox5.Height))
            f.WriteLine(TabString("GroupBox6_Height", Form1.GroupBox6.Height))
            '
            f.WriteLine(TabString("BandsCount", Form1.txt_BandsCount.Text))
            f.WriteLine(TabString("SpecMaxDb", Form1.txt_SpecMaxDb.Text))
            f.WriteLine(TabString("SpecMinDb", Form1.txt_SpecMinDb.Text))
            f.WriteLine(TabString("SpecMaxFreq", Form1.txt_SpecMaxFreq.Text))
            f.WriteLine(TabString("SpecMinFreq", Form1.txt_SpecMinFreq.Text))
            f.WriteLine(TabString("SpecSpeed", Form1.txt_SpecSpeed.Text))
            f.WriteLine(TabString("Site", Form1.txt_Site.Text))
            f.WriteLine(TabString("SpecLog", Form1.chk_SpecLog.Checked))
            f.WriteLine(TabString("Spec_Window", Form1.cmb_Spec_Window.Text))
            f.WriteLine(TabString("Spec_Samples", Form1.cmb_Samples.Text))
            '
            f.WriteLine(TabString("FirstSlot", Form1.txt_FirstSlot.Text))
            f.WriteLine(TabString("WriteToFile", Form1.chk_WriteToFile.Checked))
            '
            f.WriteLine("")
            f.WriteLine(" Calibrator")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("CalibOutputDB", Form1.txt_CalibOutputDB.Text))
            f.WriteLine(TabString("CalibEnableOut", Form1.chk_CalibEnableOut.Checked))
            f.WriteLine(TabString("CalibSelectedBand", CalibSelectedBand))
            '
            f.WriteLine("")
            f.WriteLine(" Calibrations")
            f.WriteLine("===========================================")
            f.WriteLine(GetCalibrationString())
            '
            f.WriteLine("")
            f.WriteLine("==============================================")
            f.WriteLine(" Special controls (without user interface)")
            f.WriteLine("----------------------------------------------")
            f.WriteLine("  1) Close the application")
            f.WriteLine("  2) Change special controls values")
            f.WriteLine("  3) Restart the application")
            f.WriteLine("==============================================")
            f.WriteLine(TabString("ZeroCalibration", ZeroCalibration.ToString))
            f.WriteLine(TabString("MeterSamplesMillisec", MeterSamplesMillisec.ToString) + " (10/20/50/100)")

        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub



    Friend Sub Load_INI()
        ' ------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = CInt(Val(l))
                    Case "Form1_Left" : Form1.Left = CInt(Val(l))
                    Case "Form1_Width" : Form1.Width = CInt(Val(l))
                    Case "Form1_Height" : Form1.Height = CInt(Val(l))
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                    Case "SplitterDistance" : Form1.SplitContainer1.SplitterDistance = CInt(Val(l))
                        ' ----------------------------------------------------------------- 
                    Case "SelectedAudioIn" : Form1.SelectedAudioIn = Val_Int(l)
                    Case "SelectedAudioOut" : Form1.SelectedAudioOut = Val_Int(l)
                        ' ---------------------------------------------------------------- controls
                    Case "Calibration" : Form1.txt_Calibration.Text = l
                    Case "RoundDisplayFreq" : Form1.chk_RoundDisplayFreq.Checked = l = "True"
                        '
                    Case "GroupBox1_Height" : Form1.GroupBox1.Height = Val_Int(l)
                    Case "GroupBox2_Height" : Form1.GroupBox2.Height = Val_Int(l)
                    Case "GroupBox3_Height" : Form1.GroupBox3.Height = Val_Int(l)
                    Case "GroupBox4_Height" : Form1.GroupBox4.Height = Val_Int(l)
                    Case "GroupBox5_Height" : Form1.GroupBox5.Height = Val_Int(l)
                    Case "GroupBox6_Height" : Form1.GroupBox6.Height = Val_Int(l)
                        '
                    Case "BandsCount" : Form1.txt_BandsCount.Text = l
                    Case "SpecMaxDb" : Form1.txt_SpecMaxDb.Text = l
                    Case "SpecMinDb" : Form1.txt_SpecMinDb.Text = l
                    Case "SpecMaxFreq" : Form1.txt_SpecMaxFreq.Text = l
                    Case "SpecMinFreq" : Form1.txt_SpecMinFreq.Text = l
                    Case "SpecSpeed" : Form1.txt_SpecSpeed.Text = l
                    Case "SpecLogX" : Form1.chk_SpecLog.Checked = l = "True"
                    Case "Spec_Window" : Form1.cmb_Spec_Window.Text = l
                    Case "Spec_Samples" : Form1.cmb_Samples.Text = l
                    Case "Site" : Form1.txt_Site.Text = l
                    Case "FirstSlot" : Form1.txt_FirstSlot.Text = l
                    Case "WriteToFile" : Form1.chk_WriteToFile.Checked = l = "True"
                        '
                        ' ---------------------------------------------------------------- Calibrator
                    Case "CalibOutputDB" : Form1.txt_CalibOutputDB.Text = l
                    Case "CalibEnableOut" : Form1.chk_CalibEnableOut.Checked = l = "True"
                    Case "CalibSelectedBand" : CalibSelectedBand = Val_Int(l)
                        ' 
                        ' ---------------------------------------------------------------- Calibrations
                    Case ("CalibGen10") : SetCalibrationsFromString("CalibGen10", l)
                    Case ("CalibGen31") : SetCalibrationsFromString("CalibGen31", l)
                    Case ("CalibMic10") : SetCalibrationsFromString("CalibMic10", l)
                    Case ("CalibMic31") : SetCalibrationsFromString("CalibMic31", l)
                        '
                        ' ---------------------------------------------------------------- special controls
                    Case "ZeroCalibration" : ZeroCalibration = CSng(Val(l))
                    Case "MeterSamplesMillisec" : MeterSamplesMillisec = CInt(Val(l))

                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub

End Module
